% Monetary Regime-Switches and Central Bank Preferences 
% Debortoli and Nunes (JMCB)
%
% MAIN PROGRAM: it solves and simulate the model for the following cases:
% 1 - Full Commitment
% 2 - Loose Commitment
% 3 - Regime Switches

% It calls the functions:
% - solve_model.m:  file to solve the linear model with GENSYS
% - Pol_FixPoint.m: function defining the residuals for the Fix point iteration in the Regime Switching model;
% - do_statistics.m: it calculates statistics
% - do_simul.m:      it calculates random simulations
% - impresp_sims:    it calculates impulse responses
% - show_table:      is displays a table of statistics on the screen


clear all;
clc;
global betta sigma rho kappa rho_u prob_COM

%% SET THE PARAMETERS (from Gali (2008))
% STRUCTURAL
betta=.99;    %Discount Factor
alf=1/3;      %1-Labor Productivity
sigma=1;      %Risk Aversion
phi=1;        %Elasticity of labor supply
theta=2/3;    %Price stickyness
epsilon=6;    %Elasticity of substitution (from Dixit-Stiglitz)

% SHOCKS
rho_u = 0.2;   % Persistence of cost-push shocks
%rho_u = 0.99;   % Persistence of cost-push shocks

sigma_epsu=1;  % std dev of cost-push shock (in percentage)
VCV_eps = (sigma_epsu).^2;

% AGGREGATIONS OF PARAMETERS (NOT TO CHANGE!!!)
rho=-log(betta);                                          %natural real interest rate in steady state
% parameters for NK Phillips Curve
THETA=(1-alf)/(1-alf+alf*epsilon);
lambda=THETA*(1-theta)*(1-betta*theta)/theta;
kappa=lambda*(sigma+(phi+alf)/(1-alf));

% CENTRAL BANK PREFERENCES
w_c = kappa/ epsilon ; % Hawkish (Conservative): Household base preferences
w_l =.5;               % Dovish (Liberal): Arbitary factor more than the Conservative
ybar

prob_COMch = .9;       % regime-switching probability

pol_parameters.c = [w_c];
pol_parameters.l = [w_l];

%% OPTIONS FOR STATISTICS AND IMPULSE RESPONSES
irmax     = 12;     % horizon for impulse responses
num_simul = 1000;   % number of histories in simulations
num_per   = 100;    % numbr of periods in simulations
nvar = 4;      % number of variables (endogenous + persistent shocks)
nshocks= 1;    % number of i.i.d. shocks

stats_while_in_power_by_simul=0; % set to 1 to do simulation while in power, and 0 otherwise. 

vlab    = ['Output-Gap (%)  ';...
           'Inflation (%)   ';...
           'Interest Rate   ';...
           'Cost-Push Shock '];
       
slab    = ['Cost Push Shock'];

case_lab = char('Com','LooseCom','Reg. Sw.');


scale_shocks    = [.2];             % size of the initial shock
scale_y         = [1;4;4;1];        % annualized inflation, and interest rate
irshocks        = [1];              % to which shock plot the IR
irsel{1}        = [2;1];            % which variable to show for the 1st shock
showcase        = [1;2;3];

%% initialize some values
CoeffB = zeros(20,1);  % policy parameters of alternative regime;

%% 1 - SOLUTION UNDER FULL COMMITMENT OPTIMAL
prob_COM = 1;       %Probability of commitment

% Conservative
[G1.c{1},C.c{1},impact.c{1}] = solve_model(pol_parameters.c,CoeffB);

% Liberal
[G1.l{1},C.l{1},impact.l{1}] = solve_model(pol_parameters.l,CoeffB);

%% LOOSE COMMITMENT
prob_COM = prob_COMch;

% Solve the model under the conservative regime
CoeffGuess = [[impact.c{1}; G1.c{1}(:)]; [impact.l{1}; G1.l{1}(:)]]; %guess for coefficients
CoeffFixP = fsolve(@Pol_FixPoint,CoeffGuess,[], pol_parameters.c, pol_parameters.c);

% recover coefficients
CoeffB = CoeffFixP(1:length(CoeffFixP)/2);
[G1.c{2},C.c{2},impact.c{2}] = solve_model(pol_parameters.c,CoeffB);

% Solve the model under the liberal regime
CoeffGuess = [[impact.l{1}; G1.l{1}(:)]; [impact.c{1}; G1.c{1}(:)]];
CoeffFixP = fsolve(@Pol_FixPoint,CoeffGuess,[], pol_parameters.l, pol_parameters.l);
CoeffB = CoeffFixP(1:length(CoeffFixP)/2);

% recover coefficients
[G1.l{2},C.l{2},impact.l{2}] = solve_model(pol_parameters.l,CoeffB);

%% REGIME SWITCHES (OPTIMAL)
prob_COM = prob_COMch;
RULEtype = 'optimal';
CoeffGuess = [[impact.c{1}; G1.c{1}(:)]; [impact.l{1}; G1.l{1}(:)]];

CoeffFixP = fsolve(@Pol_FixPoint,CoeffGuess,[], pol_parameters.c, pol_parameters.l);
CoeffB1 = CoeffFixP(1:length(CoeffFixP)/2);
CoeffB2 = CoeffFixP(length(CoeffFixP)/2+1:end);
[G1.c{3},C.c{3},impact.c{3}] = solve_model(pol_parameters.c,CoeffB2);
[G1.l{3},C.l{3},impact.l{3}] = solve_model(pol_parameters.l,CoeffB1);


%% DO STATISTICS AND IMPULSE RESPONSES
% single compute IR (conditional on commitment)
for k=1:3
    Resp_mat.c{k} = impresp_sims(G1.c{k},impact.c{k},irmax+1,scale_shocks,scale_y); 
    Resp_mat.l{k} = impresp_sims(G1.l{k},impact.l{k},irmax+1,scale_shocks,scale_y); 
end

%% SECOND MOMENTS and WELFARE

% G1sM_A     = G1.c{1}(1:nvar,1:nvar);
% impactsM_A = impact.c{1}(1:nvar,:);
% prob_COM=1;
% G1sM_B=G1sM_A;
% impactsM_B=impactsM_A;
% do_statistics;
% xstdev.c{1} = yxsd;
% 
% G1sM_A     = G1.l{1}(1:nvar,1:nvar);
% impactsM_A = impact.l{1}(1:nvar,:);
% prob_COM=1;
% G1sM_B=G1sM_A;
% impactsM_B=impactsM_A;
% do_statistics;
% xstdev.l{1} = yxsd;


% conservative regime always in power
for k=1:3
    % SECOND MOMENTS
    G1sM_A     = G1.c{k}(1:nvar,1:nvar);
    impactsM_A = impact.c{k}(1:nvar,:);   
    switch k
        case{1} 
            prob_COM=1;
            G1sM_B=G1sM_A;
            impactsM_B=impactsM_A;
        case{2}
            prob_COM= prob_COMch;
            G1sM_B=G1sM_A;
            impactsM_B=impactsM_A;
        case{3};
            prob_COM=prob_COMch;
            G1sM_B=G1.l{k}(1:nvar,1:nvar);
            impactsM_B =impact.l{k}(1:nvar,:);
    end
    
    if stats_while_in_power_by_simul==1;
        do_simul;
        Loss.c{k} = (w_c*yxsd(1).^2+yxsd(2).^2); 
    else
        do_statistics;
        Loss.c{k}=(w_c*yxsd(1).^2+yxsd(2).^2);
    end
    xstdev.c{k} = yxsd;
    xvariance.c{k} = yxsd.^2;
end 
 
%liberal regime always in power
for k=1:3
    % SECOND MOMENTS
    G1sM_A =G1.l{k}(1:nvar,1:nvar);
    impactsM_A =impact.l{k}(1:nvar,:);   
    switch k
        case{1} 
            prob_COM=1;
            G1sM_B=G1sM_A;
            impactsM_B=impactsM_A;
        case {2}
            prob_COM= prob_COMch;
            G1sM_B=G1sM_A;
            impactsM_B=impactsM_A;
        case {3};
            prob_COM=prob_COMch;
            G1sM_B=G1.c{k}(1:nvar,1:nvar);
            impactsM_B =impact.c{k}(1:nvar,:);
    end
    
    if stats_while_in_power_by_simul==1;
        do_simul;
        Loss.l{k} =(w_l*yxsd(1).^2+yxsd(2).^2);
    else
        do_statistics;
        Loss.l{k}=(w_l*yxsd(1).^2+yxsd(2).^2);
    end
    xstdev.l{k} = yxsd;
    xvariance.l{k} = yxsd.^2;
end 
   
% show table
disp(' ')
disp('Hawkish')
disp('***************************')
disp('St.dev')
disp('-------------------')
show_table(xstdev.c, vlab, case_lab, showcase);
show_table(Loss.c, 'Loss           ', case_lab, showcase);

disp(' ')
disp('Dovish:')
disp('***************************')
disp('St.dev')
disp('-------------------')
show_table(xstdev.l, vlab, case_lab, showcase);
show_table(Loss.l, 'Loss           ', case_lab, showcase);

%% plot Impulse Responses
nshocks=length(irshocks);
hor=[0:irmax];

ncases = length(showcase);

for shock=1:nshocks
     idshock = irshocks(shock);
     sel=irsel{idshock};
     nir=length(sel);
     
     %figure('Name',strcat('Conservative: Effects of a',slab(idshock,:)));     
     for j=1:nir
         for k=1:ncases
             yxirx.c(k,:)=Resp_mat.c{showcase(k)}(sel(j),:,idshock);
             yxirx.l(k,:)=Resp_mat.l{showcase(k)}(sel(j),:,idshock);
             legendgraph(k,:) = case_lab(showcase(k),:);
         end
         
         figure(100+shock)%'Name',strcat('Conservative: Effects of a',slab(idshock,:)));
         
         subplot(2,2,j);
 %        set(gca,'NextPlot','add','LineStyleOrder','-|--|.-',...
 %            'ColorOrder',[1 0 0; 0 1 0; 0 0 1]);
         plot(hor,yxirx.c(1,:),'.-b',hor,yxirx.c(2,:),'-r',hor,yxirx.c(3,:),'--g');
         axis([-0.5 irmax -Inf Inf]);
         axis 'auto y';
         grid on
         xlabel(vlab(sel(j),:));
         title('Hawkish Regime');
         
         subplot(2,2,j+2);         
         plot(hor,yxirx.l(1,:),'.-b',hor,yxirx.l(2,:),'-r',hor,yxirx.l(3,:),'--g');
         axis([-0.5 irmax -Inf Inf]);
         axis 'auto y';
         grid on
         xlabel(vlab(sel(j),:));
         title('Dovish Regime');
         
         clear yxirx
         %hold on
     end
     legend(legendgraph);
end

figure(200)
subplot(2,2,1)
plot(hor, Resp_mat.c{2}(2,:,1)-Resp_mat.c{1}(2,:,1),'o-k',hor,Resp_mat.c{3}(2,:,1)-Resp_mat.c{2}(2,:,1),'+-')
xlabel('Inflation (%)');
axis([-0.5 irmax -Inf Inf]);
axis 'auto y';
grid on;

subplot(2,2,2)
plot(hor, Resp_mat.c{2}(1,:,1)-Resp_mat.c{1}(1,:,1),'o-k',hor,Resp_mat.c{3}(1,:,1)-Resp_mat.c{2}(1,:,1),'+-')
xlabel('Output-Gap (%)');
axis([-0.5 irmax -Inf Inf]);
axis 'auto y';
grid on;
title('Differences in Impulse Responses: Hawkish regime')

subplot(2,2,3)
plot(hor,Resp_mat.l{2}(2,:,1)-Resp_mat.l{1}(2,:,1),'o-k',hor,Resp_mat.l{3}(2,:,1)-Resp_mat.l{2}(2,:,1),'+-')
xlabel('Inflation (%)');
axis([-0.5 irmax -Inf Inf]);
axis 'auto y';
grid on;

subplot(2,2,4)
plot(hor, Resp_mat.l{2}(1,:,1)-Resp_mat.l{1}(1,:,1),'o-k',hor,Resp_mat.l{3}(1,:,1)-Resp_mat.l{2}(1,:,1),'+-')
xlabel('Output-Gap (%)');
axis([-0.5 irmax -Inf Inf]);
axis 'auto y';
grid on;
title('Differences in Impulse Responses: Dovish regime')

legend('Limited Commitment - Full Commitment','Regimes Switches - Limited Commitment')


